@@ -1,13 +1,8 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
  * ipaddress.c		"ip address".
  *
- *		This program is free software; you can redistribute it and/or
- *		modify it under the terms of the GNU General Public License
- *		as published by the Free Software Foundation; either version
- *		2 of the License, or (at your option) any later version.
- *
  * Authors:	Alexey Kuznetsov, <kuznet@ms2.inr.ac.ru>
- *
  */
 
 #include <stdarg.h>
@@ -1269,6 +1264,18 @@ int print_linkinfo(struct nlmsghdr *n, void *arg)
 				   "gro_max_size %u ",
 				   rta_getattr_u32(tb[IFLA_GRO_MAX_SIZE]));
 
+		if (tb[IFLA_GSO_IPV4_MAX_SIZE])
+			print_uint(PRINT_ANY,
+				   "gso_ipv4_max_size",
+				   "gso_ipv4_max_size %u ",
+				   rta_getattr_u32(tb[IFLA_GSO_IPV4_MAX_SIZE]));
+
+		if (tb[IFLA_GRO_IPV4_MAX_SIZE])
+			print_uint(PRINT_ANY,
+				   "gro_ipv4_max_size",
+				   "gro_ipv4_max_size %u ",
+				   rta_getattr_u32(tb[IFLA_GRO_IPV4_MAX_SIZE]));
+
 		if (tb[IFLA_PHYS_PORT_NAME])
 			print_string(PRINT_ANY,
 				     "phys_port_name",
